#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <arpa/inet.h> //inet_ntoa

#define SERVER_PORT 12138
#define LISTEN_QUEUE 20
#define BUFFER_SIZE 1024

void print_time(char *ch);//打印时间

int main(int argc,char *argv[])
{
    struct sockaddr_in server_addr;
    bzero(&server_addr,sizeof(server_addr));
    server_addr.sin_family=AF_INET;
    server_addr.sin_addr.s_addr=htons(INADDR_ANY);
    server_addr.sin_port=htons(SERVER_PORT);

    //创建套接字
    int sockfd=socket(AF_INET,SOCK_STREAM,0);
    if(sockfd<0)
    {
        perror("创建套接字失败");
        exit(-1);
    }

    if(bind(sockfd,(struct sockaddr *)&server_addr,sizeof(server_addr))==-1)
    {
        perror("bind 失败");
        exit(-1);
    }

    if(listen(sockfd,LISTEN_QUEUE))
    {
        perror("listen 失败");
        exit(-1);
    }

    while(1)
    {
        pid_t pid;
        struct sockaddr_in client_addr;
        socklen_t length=sizeof(client_addr);
        int clientfd=accept(sockfd,(struct sockaddr *)&client_addr,&length);
        if(clientfd==-1)
        {
            perror("accept 失败");
            continue;
        }
        else
        {
            printf("客户端%s:%d连接成功\n",inet_ntoa(client_addr.sin_addr),ntohs(client_addr.sin_port));
            pid=fork();
            if(pid<0)
            {
                perror("创建进程失败");
            }
            else if(pid==0)/*child*/
            {
                char buffer[BUFFER_SIZE];
                int data_len;
                FILE * fp=NULL;
                bzero(buffer,BUFFER_SIZE);
                if((fp=fopen("data","wb"))==NULL)
                {
                    perror("文件打开失败");
                    exit(-1);
                }
                //循环接收数据
                int size=0;//表示有多少个块
                while(data_len=recv(clientfd,buffer,BUFFER_SIZE,0))//data_len为0时结束，是因为当客户端没有再发送数据过来时是接收0的，也表示该文件传输完毕了。
                {
                    if(data_len<0)
                    {
                        perror("接收数据错误");
                        exit(-1);
                    }
                    size++;
                    if(size==1)
                    {
                        printf("正在接收来自%s:%d的文件\n",inet_ntoa(client_addr.sin_addr),ntohs(client_addr.sin_port));
                    }
                    else
                    {
                        printf(".");
                    }
                    //向文件中写入
                    int write_len=fwrite(buffer,sizeof(char),data_len,fp);//向文件中写入，默认文件打开时会有一个文件指针进行写入。如果是并行传输就要修改这个文件指针了，还是有点麻烦的，如果是串行的，我们都不用管了，多方便。
                    if(write_len>data_len)
                    {
                        perror("写入数据错误");
                        exit(-1);
                    }
                    bzero(buffer,BUFFER_SIZE);
                }
                if(size>0)
                {
                    printf("\n%s:%d的文件传送完毕\n",inet_ntoa(client_addr.sin_addr),ntohs(client_addr.sin_port));
                }
                else
                {
                    printf("\n%s:%d的文件传送失败\n",inet_ntoa(client_addr.sin_addr),ntohs(client_addr.sin_port));//如果传过来的文件大小为0，也是会出现这个错误
                }
                fclose(fp);
                //rename("data","asdf");//这里可以修改文件的名字。保存到服务器的话可以随便改个名字，防止文件名重复
                exit(0);
            }
            else /*pather*/
            {
                ;
            }
        }
        close(clientfd);
    }

    return 0;
}

void print_time(char *ch)
{
    time_t now;
    struct tm * stm;
    time(&now);
    stm=localtime(&now);
    sprintf(ch,"%02d:%02d:%02d\n",stm->tm_hour,stm->tm_min,stm->tm_sec);
    return ;
}
