#include <netinet/in.h> // sockaddr_in
#include <sys/types.h> //socket
#include <sys/socket.h> //socket
#include <netdb.h> //gethostbyname
#include <unistd.h> //close
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <arpa/inet.h> //inet_addr

#define SERVVER_PORT 12138
#define LISTEN_QUEUE 20
#define BUFFER_SIZE 1024

//传输控制信号宏定义
//struct Control中control的取值
#define USER_CHECK_LOGIN 1
#define FILE_PUSH 2
#define FILE_PULL 3
#define FILE_LIST 4
#define FILE_SENDTO 5
#define FILE_DELECT 6

struct Addr
{
    char host[64];
    int port;
};
struct User
{
    int uid;
    char username[64];
    char password[64];
};
struct Control
{
    int uid;
    int control;
};

void print_time(char *ch);//打印时间
int file_push(struct Addr addr,struct User user,char *filenames);
int check_login(struct Addr addr,struct User * user);

int main(int argc,char *argv[])
{
    char orderbuf[BUFFER_SIZE];
    struct Addr addr;
    char arg1[32],arg2[32],arg3[32],arg4[32];
    struct User user;

    if(argc!=5)
    {
        perror("usage: ./client [serverhost] [serverport] [username] [password]");
        exit(-1);
    }
    strcpy(addr.host,argv[1]);
    addr.port=atoi(argv[2]);
    strcpy(user.username,argv[3]);
    strcpy(user.password,argv[4]);
    int uid=check_login(addr,&user);
    if(uid<=0)
    {
        perror("用户验证失败");
        exit(-1);
    }
    printf("验证登陆成功\n");
    while(1)
    {
        printf("\n请输入操作指令:");
        fgets(orderbuf,BUFFER_SIZE,stdin);
        memset(arg1,0,sizeof(arg1));
        memset(arg2,0,sizeof(arg2));
        memset(arg3,0,sizeof(arg3));
        memset(arg4,0,sizeof(arg4));
        sscanf(orderbuf,"%s%s%s%s",arg1,arg2,arg3,arg4);
        if(strcmp(arg1,"file")==0)
        {
            if(strcmp(arg2,"push")==0)
            {
                strcpy(orderbuf,arg3);
                file_push(addr,user,orderbuf);
            }
            else if(strcmp(arg2,"pull")==0)
            {
                ;
            }
            else if(strcmp(arg2,"list")==0)
            {
                ;
            }
            else if(strcmp(arg2,"sendto")==0)
            {
                ;
            }
            else if(strcmp(arg2,"delect")==0)
            {
                ;
            }
            else
            {
                printf("该命令不支持\n");
            }
        }
        else if(strcmp(arg1,"quit")==0 || strcmp(arg1,"logout")==0)
        {
            break;
        }
        else 
        {
            printf("该命令不支持\n");
        }
    }


    return 0;
}

//验证成功时返回大于0的uid号码，错误返回-1
int check_login(struct Addr addr,struct User * user)
{
    struct sockaddr_in servAddr;
    struct hostent * host;
    struct Control control;
    int sockfd;

    host=gethostbyname(addr.host);
    servAddr.sin_family=AF_INET;
    servAddr.sin_addr=*((struct in_addr *)host->h_addr);
    //servAddr.sin_addr.s_addr=inet_addr("127.0.0.1");
    servAddr.sin_port=htons(addr.port);
    if(host==NULL)
    {
        perror("获取IP地址失败");
        exit(-1);
    }
    if((sockfd=socket(AF_INET,SOCK_STREAM,0))==-1)
    {
        perror("socket创建失败");
        exit(-1);
    }
    if(connect(sockfd,(struct sockaddr *)&servAddr,sizeof(struct sockaddr_in))==-1)
    {
        perror("connect 失败");
        exit(-1);
    }

    control.control=USER_CHECK_LOGIN;
    control.uid=0;
    if(send(sockfd,(char *)&control,sizeof(struct Control),0)<0)
    {
        perror("发送数据失败");
        exit(-1);
    }

    if(send(sockfd,(char *)user,sizeof(struct User),0)<0)
    {
        perror("发送数据失败");
        exit(-1);
    }
    bzero(user,sizeof(struct User));
    if(recv(sockfd,(char *)user,sizeof(struct User),0)<0)
    {
        perror("接收数据失败");
        exit(-1);
    }
    printf("获取后用户名:%s 密码:%s ID号:%d\n",user->username,user->password,user->uid);

    close(sockfd);//关闭socket连接
    return user->uid;
}

int file_push(struct Addr addr,struct User user,char *filenames)
{
    struct sockaddr_in servAddr;
    struct hostent * host;
    struct Control control;
    int sockfd;
    FILE *fp;

    host=gethostbyname(addr.host);
    servAddr.sin_family=AF_INET;
    servAddr.sin_addr=*((struct in_addr *)host->h_addr);
    //servAddr.sin_addr.s_addr=inet_addr("127.0.0.1");
    servAddr.sin_port=htons(addr.port);
    if(host==NULL)
    {
        perror("获取IP地址失败");
        exit(-1);
    }
    if((sockfd=socket(AF_INET,SOCK_STREAM,0))==-1)
    {
        perror("socket创建失败");
        exit(-1);
    }

    if(connect(sockfd,(struct sockaddr *)&servAddr,sizeof(struct sockaddr_in))==-1)
    {
        perror("connect 失败");
        exit(-1);
    }

    //打开文件
    if((fp=fopen(filenames,"rb"))==NULL)
    {
        perror("文件打开失败");
        exit(-1);
    }
    //这里传输控制信号
    control.control=FILE_PUSH;
    control.uid=user.uid;
    if(send(sockfd,(char *)&control,sizeof(struct Control),0)<0)
    {
        perror("控制信号发送失败");
        exit(-1);
    }
    char buffer[BUFFER_SIZE];
    bzero(buffer,BUFFER_SIZE);
    printf("正在传输文件");
    int len=0;
    //不断的读取文件直到文件结束
    while((len=fread(buffer,1,BUFFER_SIZE,fp))>0)
    {
        if(send(sockfd,buffer,len,0)<0)
        {
            perror("发送数据失败");
            exit(-1);
        }
        bzero(buffer,BUFFER_SIZE);
        printf(".");//1K打印一个点//如果要实现百分比，就要计算文件大小，然后再处理即可
    }

    printf("传输完毕\n");
    fclose(fp);//关闭文件流
    close(sockfd);//关闭socket连接

    return 0;
}

void print_time(char *ch)
{
    time_t now;
    struct tm * stm;
    time(&now);
    stm=localtime(&now);
    sprintf(ch,"%02d:%02d:%02d\n",stm->tm_hour,stm->tm_min,stm->tm_sec);
    return ;
}
