#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <netdb.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <arpa/inet.h> //inet_ntoa
#include <mysql.h>

#define SERVER_PORT 12138
#define LISTEN_QUEUE 20
#define BACKLOG 200
#define BUFFER_SIZE 1024

//传输控制信号宏定义
//struct Control中control的取值
#define USER_CHECK_LOGIN 1
#define FILE_PUSH 2
#define FILE_PULL 3
#define FILE_LIST 4
#define FILE_SENDTO 5
#define FILE_DELETE 6

struct User
{
    int uid;
    char username[64];
    char password[64];
};
struct Control
{
    int uid;
    int control;
};
struct File
{
    int uid;
    char filename[64];
    char md5[64];
};
struct FileList
{
    int cnt;
    char list[16][128];
};

void print_time(char *ch);//打印时间
int MAX(int a,int b);
int mysql_check_login(struct User user);
int mysql_file_in(struct File file);
int mysql_file_in(int uid,int fid);
int mysql_get_max_fid();
int mysql_check_md5(struct File file);
int mysql_get_file_list(struct File file,struct FileList *filelist);
int mysql_get_md5_from_file(struct File * file);
int mysql_delete_file(struct File file);
int mysql_sendto(struct File file,struct User to);






int main(int argc,char *argv[])
{
    struct sockaddr_in server_addr;
    struct sockaddr_in client_addr;
    struct User user;
    struct Control control;
    char ch[64];
    int clientfd;
    pid_t pid;
    socklen_t length;
    bzero(&server_addr,sizeof(server_addr));
    server_addr.sin_family=AF_INET;
    server_addr.sin_addr.s_addr=htons(INADDR_ANY);
    server_addr.sin_port=htons(SERVER_PORT);

    //创建套接字
    int sockfd=socket(AF_INET,SOCK_STREAM,0);
    if(sockfd<0)
    {
        perror("创建套接字失败");
        exit(-1);
    }

    if(bind(sockfd,(struct sockaddr *)&server_addr,sizeof(server_addr))==-1)
    {
        perror("bind 失败");
        exit(-1);
    }

    if(listen(sockfd,LISTEN_QUEUE))
    {
        perror("listen 失败");
        exit(-1);
    }

    length=sizeof(struct sockaddr);
    while(1)
    {
        clientfd=accept(sockfd,(struct sockaddr *)&client_addr,&length);
        if(clientfd==-1)
        {
            perror("accept 失败");
            continue;
        }
        printf(">>>>>%s:%d 连接成功，当前所在的ID(fd)号: %d \n",inet_ntoa(client_addr.sin_addr),ntohs(client_addr.sin_port),clientfd);
        print_time(ch);
        printf("加入的时间是:%s\n",ch);

        //来一个连接就创建一个进程进行处理
        pid=fork();
        if(pid<0)
        {
            perror("fork error");
        }
        else if(pid==0)
        {
            recv(clientfd,(char *)&control,sizeof(struct Control),0);
            printf("用户 %d 使用命令 %d\n",control.uid,control.control);
            switch(control.control)
            {
                case USER_CHECK_LOGIN:
                    {
                        //身份验证处理
                        recv(clientfd,(char *)&user,sizeof(struct User),0);
                        printf("客户端发送过来的用户名是:%s,密码:%s\n",user.username,user.password);
                        if((user.uid=mysql_check_login(user))>0)
                        {
                            printf("验证成功\n");
                        }
                        else
                        {
                            printf("验证失败\n");
                        }
                        send(clientfd,(char *)&user,sizeof(struct User),0);
                        break;
                    }
                case FILE_PUSH:
                    {
                        char buffer[BUFFER_SIZE];
                        int data_len;
                        FILE * fp=NULL;
                        struct File file;
                        //获取文件指纹
                        recv(clientfd,(char *)&file,sizeof(struct File),0);
                        printf("获取到的用户名ID: %d 文件名:%s  MD5:%s\n",file.uid,file.filename,file.md5);
                        //对文件进行验证，如果文件已经存在就不用进行接收了
                        int t=mysql_check_md5(file);
                        char ch[64]={0};
                        printf("t=%d\n",t);
                        if(t!=0)
                        {
                            printf("该文件存在,使用秒传功能\n");
                            strcpy(ch,"yes");
                            send(clientfd,ch,64,0);
                            mysql_file_in(file.uid,t);
                            //continue;
                        }
                        strcpy(ch,"no");
                        send(clientfd,ch,64,0);
                        printf("md5验证后得到的fid:%d\n",t);
                        bzero(buffer,BUFFER_SIZE);
                        if((fp=fopen("data","wb"))==NULL)
                        {
                            perror("文件打开失败");
                            exit(-1);
                        }
                        //循环接收数据
                        int size=0;//表示有多少个块
                        while(data_len=recv(clientfd,buffer,BUFFER_SIZE,0))
                        {
                            if(data_len<0)
                            {
                                perror("接收数据错误");
                                exit(-1);
                            }
                            size++;
                            if(size==1)
                                printf("正在接收来自%s:%d的文件\n",inet_ntoa(client_addr.sin_addr),ntohs(client_addr.sin_port));
                            else
                                printf(".");
                            //向文件中写入
                            int write_len=fwrite(buffer,sizeof(char),data_len,fp);
                            if(write_len>data_len)
                            {
                                perror("写入数据错误");
                                exit(-1);
                            }
                            bzero(buffer,BUFFER_SIZE);
                        }
                        if(size>0)
                        {
                            printf("\n%s:%d的文件传送完毕\n",inet_ntoa(client_addr.sin_addr),ntohs(client_addr.sin_port));
                            //如果文件传输成功那么就可以写入数据库了
                            mysql_file_in(file);
                        }
                        else
                            printf("\n%s:%d的文件传送失败\n",inet_ntoa(client_addr.sin_addr),ntohs(client_addr.sin_port));
                        fclose(fp);
                        rename("data",file.md5);//这里可以修改文件的名字
                        exit(0);
                        break;
                    }
                case FILE_PULL:
                    {
                        struct File file;
                        char ch[64];
                        memset(ch,0,sizeof(ch));
                        //获取接下来要发送的文件
                        recv(clientfd,(char *)&file,sizeof(struct File),0);
                        //根据uid和filename获取服务器中的唯一文件，然后发送
                        int t=mysql_get_md5_from_file(&file);
                        printf("获取到的MD5:%s\n",file.md5);
                        if(t==-1||file.md5[0]==0)//服务器没有对应的文件
                        {
                            printf("没有对应的文件\n");;
                            strcpy(ch,"no");
                            send(clientfd,ch,64,0);
                            break;
                        }
                        strcpy(ch,"yes");
                        send(clientfd,ch,64,0);

                        FILE * fp = NULL;
                        if((fp=fopen(file.md5,"rb"))==NULL)
                        {
                            perror("文件打开失败");
                        }
                        char buffer[BUFFER_SIZE];
                        bzero(buffer,BUFFER_SIZE);
                        printf("正在传输文件");
                        int len=0;
                        while((len=fread(buffer,sizeof(char),BUFFER_SIZE,fp))>0)
                        {
                            if(send(clientfd,buffer,BUFFER_SIZE,0)<0)
                            {
                                perror("发送数据失败");
                            }
                            bzero(buffer,BUFFER_SIZE);
                            printf(".");
                        }
                        printf("传输完成\n");
                        fclose(fp);
                        break;
                    }
                case FILE_LIST:
                    {
                        struct File file;
                        struct FileList filelist;
                        file.uid=control.uid;
                        mysql_get_file_list(file,&filelist);
                        send(clientfd,(char *)&filelist,sizeof(struct FileList),0);
                        break;
                    }
                case FILE_DELETE:
                    {
                        struct File file;
                        char ch[64];
                        memset(ch,0,sizeof(ch));
                        recv(clientfd,(char *)&file,sizeof(struct File),0);
                        int t=mysql_delete_file(file);
                        if(t==-1)
                        {
                            printf("没有对应的文件\n");;
                            strcpy(ch,"no");
                            send(clientfd,ch,64,0);
                            break;
                        }
                        strcpy(ch,"yes");
                        send(clientfd,ch,64,0);
                        printf("删除成功\n");
                        break;
                    }
                case FILE_SENDTO:
                    {
                        struct File file;
                        struct User to;
                        recv(clientfd,(char *)&file,sizeof(struct File),0);
                        recv(clientfd,(char *)&to,sizeof(struct User),0);
                        int t=mysql_sendto(file,to);
                        char ch[64];
                        memset(ch,0,sizeof(ch));
                        if(t==-1)
                        {
                            printf("没有对应的文件\n");;
                            strcpy(ch,"no1");
                            send(clientfd,ch,64,0);
                            break;
                        }
                        else if(t==-2)
                        {
                            printf("没有对应的用户\n");;
                            strcpy(ch,"no2");
                            send(clientfd,ch,64,0);
                            break;
                        }
                        strcpy(ch,"yes");
                        send(clientfd,ch,64,0);
                        printf("共享成功\n");
                        break;
                    }
                default:
                    {
                        break;
                    }
            }
            shutdown(clientfd,2);//这里要注意加一个shutdown否则客户端接收不到结束符而一直等待接收数据。普通的close是不会发送结束符的
            close(clientfd);//短连接结束
            exit(0);//退出子进程
        }
    }

    return 0;
}


//函数定义
int mysql_sendto(struct File file,struct User to)
{
    //insert into relations values(uid,fid);
    MYSQL conn;
    MYSQL_RES * res_ptr;
    MYSQL_ROW result_row;
    int res;int row;int column;
    int i,j;int fid;int uid;int rt=0;
    char sql[256];
    char ch[64];

    mysql_init(&conn);
    if(mysql_real_connect(&conn,"localhost","root","","filetranslate",0,NULL,CLIENT_FOUND_ROWS))
    {
        //select files.fid from files,relations where relations.fid=files.fid and filename=files.filename;
        //得到fid后
        strcpy(sql,"select files.fid from files,relations where relations.fid=files.fid and filename=\"");
        strcat(sql,file.filename);
        strcat(sql,"\";");
        res=mysql_query(&conn,sql);
        fid=0;
        if(res)
        {
            perror("Select Sql Error!");
        }
        else
        {
            res_ptr=mysql_store_result(&conn);
            if(res_ptr)
            {
                column=mysql_num_fields(res_ptr);
                row=mysql_num_rows(res_ptr)+1;
                if(row<=1)
                    ;
                else
                {
                    result_row=mysql_fetch_row(res_ptr);
                    if(result_row[0]==NULL)
                    {
                        fid=0;
                    }
                    else
                    {
                        fid=atoi(result_row[0]);
                    }
                }
            }
            else
            {
                fid=0;
            }
        }
        if(fid==0)
        {
            mysql_close(&conn);
            return -1;//表示没有该文件
        }

        //select uid from users where username=[to].username;
        //得到uid后
        strcpy(sql,"select uid from users where username=\"");
        strcat(sql,to.username);
        strcat(sql,"\"");
        res=mysql_query(&conn,sql);
        uid=0;
        if(res)
        {
            perror("Select Sql Error!");
        }
        else
        {
            res_ptr=mysql_store_result(&conn);
            if(res_ptr)
            {
                column=mysql_num_fields(res_ptr);
                row=mysql_num_rows(res_ptr)+1;
                if(row<=1)
                    ;
                else
                {
                    result_row=mysql_fetch_row(res_ptr);
                    if(result_row[0]==NULL)
                    {
                        uid=0;
                    }
                    else
                    {
                        uid=atoi(result_row[0]);
                    }
                }
            }
            else
            {
                uid=0;
            }
        }
        if(uid==0)
        {
            mysql_close(&conn);
            return -2;//表示没有该用户
        }

        //将获取到的uid fid插入到数据库relations中
        //insert into relations values(uid,fid);
        strcpy(sql,"insert into relations values( ");
        sprintf(ch,"%d",uid);
        strcat(sql,ch);
        strcat(sql,", ");
        sprintf(ch,"%d",fid);
        strcat(sql,ch);
        strcat(sql,");");
        res=mysql_query(&conn,sql);
        printf("==========> uid=%d  fid=%d\n",uid,fid);
        if(res)
        {
            rt=-1;
            printf("Insert Error\n");
        }
        else
        {
            printf("Insert Success\n");
        }
    }
    else
    {
        perror("Connect Failed!");
        exit(-1);
    }
    mysql_close(&conn);
    return rt;
}

int mysql_delete_file(struct File file)
{
    MYSQL conn;
    MYSQL_RES * res_ptr;
    MYSQL_ROW result_row;
    int res;int row;int column;
    int i,j;
    char sql[256];
    char ch[64];
    int fid;int rt=0;

    mysql_init(&conn);
    if(mysql_real_connect(&conn,"localhost","root","","filetranslate",0,NULL,CLIENT_FOUND_ROWS))
    {
        //select files.fid from files,relations where relations.fid=files.fid and filename= [file].filename
        strcpy(sql,"select files.fid from files,relations where relations.fid=files.fid and filename=\"");
        strcat(sql,file.filename);
        strcat(sql,"\";");
        res=mysql_query(&conn,sql);
        fid=0;
        if(res)
        {
            perror("Select Sql Error!");
        }
        else
        {
            res_ptr=mysql_store_result(&conn);
            if(res_ptr)
            {
                column=mysql_num_fields(res_ptr);
                row=mysql_num_rows(res_ptr)+1;
                if(row<=1)
                    ;
                else
                {
                    result_row=mysql_fetch_row(res_ptr);
                    if(result_row[0]==NULL)
                    {
                        fid=0;
                    }
                    else
                    {
                        fid=atoi(result_row[0]);
                    }
                }
            }
            else
            {
                fid=0;
            }
        }
        if(fid==0)
        {
            mysql_close(&conn);
            return -1;
        }
        //根据获取到的fid然后删除relations对应fid和uid
        //delect relations where uid='uid' and fid='fid'
        strcpy(sql,"delete from relations where uid=");
        sprintf(ch,"%d",file.uid);
        strcat(sql,ch);
        strcat(sql," and fid=");
        sprintf(ch,"%d",fid);
        strcat(sql,ch);
        res=mysql_query(&conn,sql);
        if(res)
        {
            printf("Delete Error\n");
        }
        else
        {
            ;;
        }
    }
    else
    {
        perror("Connect Failed!");
        exit(-1);
    }

    mysql_close(&conn);
    return rt;
}

int mysql_check_md5(struct File file)
{
    MYSQL conn;
    MYSQL_RES * res_ptr;
    MYSQL_ROW result_row;
    int res;int row;int column;int value=0;
    char sql[256]={0};
    strcpy(sql,"select fid from files where md5='");
    strcat(sql,file.md5);
    strcat(sql,"';");
    printf("查询的sql:%s\n",sql);

    mysql_init(&conn);
    if(mysql_real_connect(&conn,"localhost","root","","filetranslate",0,NULL,CLIENT_FOUND_ROWS))
    {
        res=mysql_query(&conn,sql);
        if(res)
        {
            perror("Select Sql Error!");
            exit(-1);
        }
        else
        {
            res_ptr=mysql_store_result(&conn);
            if(res_ptr)
            {
                column=mysql_num_fields(res_ptr);
                row=mysql_num_rows(res_ptr)+1;
                if(row<=1)
                {
                    ;
                }
                else
                {
                    result_row=mysql_fetch_row(res_ptr);
                    value=atoi(result_row[0]);
                }
            }
            else
            {
                printf("没有查询到匹配的数据\n");
            }
        }
    }
    else
    {
        perror("Connect Failed!");
        exit(-1);
    }
    mysql_close(&conn);
    return value;//返回fid
}

int mysql_file_in(int uid,int fid)
{
    MYSQL conn;
    int res;
    char sql[256];
    char tmp[32];
    mysql_init(&conn);
    if(mysql_real_connect(&conn,"localhost","root","","filetranslate",0,NULL,CLIENT_FOUND_ROWS))
    {
        //insert into files values(uid,fid);
        memset(sql,0,sizeof(sql));
        strcpy(sql,"insert into relations values(");
        sprintf(tmp,"%d",uid);
        strcat(sql,tmp);
        strcat(sql,",");
        sprintf(tmp,"%d",fid);
        strcat(sql,tmp);
        strcat(sql,");");
        printf("插入的sql语句: %s\n",sql);
        res=mysql_query(&conn,sql);
        if(res)
            printf("Insert Error!\n");
        else
            printf("Insert Success!\n");
    }
    else
    {
        perror("Connect Failed!");
        exit(-1);
    }
    return 0;
}

int mysql_file_in(struct File file)
{
    int mfid;
    MYSQL conn;
    int res;
    char sql[256];
    char tmp[32];
    mfid=mysql_get_max_fid()+1;
    printf("获取到的最大fid为:%d\n",mfid);
    mysql_init(&conn);
    if(mysql_real_connect(&conn,"localhost","root","","filetranslate",0,NULL,CLIENT_FOUND_ROWS))
    {
        //insert into files values(mfid,file.filename,file.md5);
        //insert into files values(file.uid,mfid);
        memset(sql,0,sizeof(sql));
        strcpy(sql,"insert into files values(");
        sprintf(tmp,"%d",mfid);
        strcat(sql,tmp);
        strcat(sql,",'");
        strcat(sql,file.filename);
        strcat(sql,"','");
        strcat(sql,file.md5);
        strcat(sql,"');");
        printf("插入的sql语句: %s\n",sql);
        res=mysql_query(&conn,sql);
        if(res)
            printf("Insert Error!\n");
        else
            printf("Insert Success!\n");

        memset(sql,0,sizeof(sql));
        strcpy(sql,"insert into relations values(");
        sprintf(tmp,"%d",file.uid);
        strcat(sql,tmp);
        strcat(sql,",");
        sprintf(tmp,"%d",mfid);
        strcat(sql,tmp);
        strcat(sql,");");
        printf("插入的sql语句: %s\n",sql);
        res=mysql_query(&conn,sql);
        if(res)
            printf("Insert Error!\n");
        else
            printf("Insert Success!\n");
    }
    else
    {
        perror("Connect Failed!");
        exit(-1);
    }
    return 0;
}

int mysql_check_login(struct User user)
{
    MYSQL conn;
    MYSQL_RES *res_ptr;
    MYSQL_ROW result_row;
    int res;
    int row;
    int column;
    int uid;
    char sql[256]={0};
    strcpy(sql,"select uid from users where username=\"");
    strcat(sql,user.username);
    strcat(sql,"\" and password=\"");
    strcat(sql,user.password);
    strcat(sql,"\";");
    printf("查询的sql:%s\n",sql);
    uid=-1;
    mysql_init(&conn);
    if(mysql_real_connect(&conn,"localhost","root","","filetranslate",0,NULL,CLIENT_FOUND_ROWS))
    {
        res=mysql_query(&conn,sql);
        if(res)
        {
            perror("Select SQL ERROR!");
            exit(-1);
        }
        else
        {
            res_ptr=mysql_store_result(&conn);
            if(res_ptr)
            {
                column=mysql_num_fields(res_ptr);//获取列数
                row=mysql_num_rows(res_ptr)+1;//获取行数，加1表示还有第一行字段名
                if(row<=1)
                {
                    ;//验证失败
                }
                else
                {
                    //这里先假定username是唯一
                    result_row=mysql_fetch_row(res_ptr);
                    printf("获取到的uid是:%s\n",result_row[0]);
                    uid=atoi(result_row[0]);
                }
            }
            else
            {
                printf("没有查询到匹配的数据\n");
            }
        }
    }
    else
    {
        perror("Connect Failed!\n");
        exit(-1);
    }
    mysql_close(&conn);
    return uid;
}

int mysql_get_file_list(struct File file,struct FileList *filelist)
{
    MYSQL conn;
    MYSQL_RES * res_ptr;
    MYSQL_ROW result_row;
    int res;int row;int column;
    int i,j;
    char sql[256]={0};
    char ch[64];
    //select filename from files,relations where relations.uid=[file].uid and relations.fid=files.fid;
    strcpy(sql,"select filename from files,relations where relations.uid=");
    sprintf(ch,"%d",file.uid);
    strcat(sql,ch);
    strcat(sql," and relations.fid=files.fid ;");
    //printf("==>%s\n",sql);

    mysql_init(&conn);
    if(mysql_real_connect(&conn,"localhost","root","","filetranslate",0,NULL,CLIENT_FOUND_ROWS))
    {
        res=mysql_query(&conn,sql);
        if(res)
        {
            perror("select sql error1");
        }
        else
        {
            res_ptr=mysql_store_result(&conn);
            if(res_ptr)
            {
                column=mysql_num_fields(res_ptr);
                row=mysql_num_rows(res_ptr)+1;
                //按行输出结果
                filelist->cnt=row-1;
                for(i=1;i<row;i++)
                {
                    result_row=mysql_fetch_row(res_ptr);
                    strcpy(filelist->list[i-1],result_row[0]);
                    //printf("%s",result_row[0]);
                }
            }
            else
            {
                printf("没有数据\n");
            }
        }
    }
    else
    {
        perror("Connect Failed1\n");
        exit(-1);
    }
    mysql_close(&conn);
    return 0;
}

int mysql_get_md5_from_file(struct File * file)
{
    //select md5 from files,relations where files.fid=relations.fid and file.md5=file.md5;
    MYSQL conn;
    MYSQL_RES * res_ptr;
    MYSQL_ROW result_row;
    int res;int row;int column;
    int rt;
    char ch[32];
    char sql[256]={0};
    rt=0;
    strcpy(sql,"select md5 from files,relations where files.fid=relations.fid and files.filename=\"");
    strcat(sql,file->filename);
    strcat(sql,"\"");
    strcat(sql," and relations.uid=");
    sprintf(ch,"%d",file->uid);
    strcat(sql,ch);
    strcat(sql,";");
    printf("查询的sql:%s\n",sql);
    mysql_init(&conn);
    if(mysql_real_connect(&conn,"localhost","root","","filetranslate",0,NULL,CLIENT_FOUND_ROWS))
    {
        res=mysql_query(&conn,sql);
        if(res)
        {
            perror("select sql error!");
        }
        else
        {
            res_ptr=mysql_store_result(&conn);
            if(res_ptr)
            {
                column=mysql_num_fields(res_ptr);
                row=mysql_num_rows(res_ptr)+1;
                if(row<=1)
                {
                    ;
                }
                else
                {
                    result_row=mysql_fetch_row(res_ptr);
                    if(result_row[0]==NULL)
                    {
                        rt=-1;
                        strcpy(file->md5,"");
                    }
                    else
                        strcpy(file->md5,result_row[0]);
                }
            }
            else
            {
                rt=-1;
                printf("没有数据\n");
            }
        }
    }
    else
    {
        perror("Connect Failed1");
        exit(-1);
    }
    mysql_close(&conn);
    return rt;
}

int mysql_get_max_fid()
{
    MYSQL conn;
    MYSQL_RES *res_ptr;
    MYSQL_ROW result_row;
    int res;int row;int column;int mfid;
    char sql[256];
    strcpy(sql,"select max(fid) from files;");
    mfid=0;
    mysql_init(&conn);
    if(mysql_real_connect(&conn,"localhost","root","","filetranslate",0,NULL,CLIENT_FOUND_ROWS))
    {
        res=mysql_query(&conn,sql);
        if(res)
        {
            perror("SELECT SQL ERROR!");
            exit(-1);
        }
        else
        {
            res_ptr=mysql_store_result(&conn);
            if(res_ptr)
            {
                column=mysql_num_fields(res_ptr);
                row=mysql_num_rows(res_ptr)+1;
                if(row<=1)
                {
                    ;//没有数据
                }
                else
                {
                    result_row=mysql_fetch_row(res_ptr);
                    printf("最大的fid是:%s\n",result_row[0]);
                    if(result_row[0]==NULL)
                        mfid=0;
                    else
                        mfid=atoi(result_row[0]);
                }
            }
            else
            {
                printf("没有查询到匹配的数据\n");
            }
        }
    }
    else
    {
        perror("Connect Failed!");
        exit(-1);
    }
    mysql_close(&conn);
    return mfid;
}

void print_time(char *ch)
{
    time_t now;
    struct tm * stm;
    time(&now);
    stm=localtime(&now);
    sprintf(ch,"%02d:%02d:%02d\n",stm->tm_hour,stm->tm_min,stm->tm_sec);
    return ;
}

int MAX(int a,int b)
{
    if(a>b)
        return a;
    return b;
}
